import 'dart:async';
import 'dart:collection';
import 'dart:io';
import 'dart:ui';

import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

import 'callbackDispatcher.dart';

typedef void MessageHandler(Map<dynamic, dynamic> bundle);

const String BACKGROUND_HANDLER = "callback_dispatch_handler";
const String INIT_SERVICE = "initService";
const String START_BACKGROUND_ISOLATE = "startBackgroundIsolate";

const String ON_PUSH_REGISTER = "onPushRegister";
const String ON_PUSH_RECEIVE = "onPushReceive";
const String ON_PUSH_RECEIVE_BACKGROUND = "onPushReceiveBackground";
const String ON_PUSH_OPEN = "onPushOpen";
const String ON_PUSH_DISMISS = "onPushDismiss";
const String ON_PUSH_BUTTON_CLICKED = "onPushButtonClicked";
const String ON_CAROUSEL_OBJECT_SELECTED = "onCarouselObjectSelected";

class NetmeraPushBroadcastReceiver {
  factory NetmeraPushBroadcastReceiver() => _instance;

  NetmeraPushBroadcastReceiver.private(MethodChannel channel)
      : _channel = channel;

  static final NetmeraPushBroadcastReceiver _instance =
  NetmeraPushBroadcastReceiver.private(
      const MethodChannel('netmera_flutter_sdk'));

  static bool _bgHandlerInitialized = false;

  final MethodChannel _channel;

  MessageHandler? _onPushRegister;
  MessageHandler? _onPushReceive;
  MessageHandler? _onPushOpen;
  MessageHandler? _onPushDismiss;
  MessageHandler? _onPushButtonClicked;
  MessageHandler? _onCarouselObjectSelected;

  static MessageHandler? _onPushReceiveBackgroundHandler;

  /// Set a message handler function which is called when the app is in the background or terminated.
  /// This provided handler must be a top-level function and cannot be anonymous.
  static onPushReceiveBackground(MessageHandler? handler) {
    _onPushReceiveBackgroundHandler = handler;

    if (handler != null) {
      _instance.registerBackgroundMessageHandler(handler);
    }
  }

  /// This function sets push action callbacks and initializes Flutter Method Channel.
  ///
  /// These callbacks are optional.
  Future<void> initialize({
    MessageHandler? onPushRegister,
    MessageHandler? onPushReceive,
    MessageHandler? onPushOpen,
    MessageHandler? onPushDismiss,
    MessageHandler? onPushButtonClicked,
    MessageHandler? onCarouselObjectSelected,
  }) async {
    WidgetsFlutterBinding.ensureInitialized();

    _onPushRegister = onPushRegister ?? _onPushRegister;
    _onPushReceive = onPushReceive ?? _onPushReceive;
    _onPushOpen = onPushOpen ?? _onPushOpen;
    _onPushDismiss = onPushDismiss ?? _onPushDismiss;
    _onPushButtonClicked = onPushButtonClicked ?? _onPushButtonClicked;
    _onCarouselObjectSelected =
        onCarouselObjectSelected ?? _onCarouselObjectSelected;

    _channel.setMethodCallHandler((call) async {
      final arguments = call.arguments;
      switch (call.method) {
        case ON_PUSH_REGISTER:
          _onPushRegister?.call(arguments);
          break;
        case ON_PUSH_RECEIVE:
          _onPushReceive?.call(arguments);
          break;
        case ON_PUSH_RECEIVE_BACKGROUND:
          _onPushReceiveBackgroundHandler?.call(arguments);
          break;
        case ON_PUSH_OPEN:
          _onPushOpen?.call(arguments);
          break;
        case ON_PUSH_DISMISS:
          _onPushDismiss?.call(arguments);
          break;
        case ON_PUSH_BUTTON_CLICKED:
          _onPushButtonClicked?.call(arguments);
          break;
        case ON_CAROUSEL_OBJECT_SELECTED:
          _onCarouselObjectSelected?.call(arguments);
          break;
      }
    });

    await _channel.invokeMethod(INIT_SERVICE);
  }

  /// This function registers background message handler to native side
  Future<void> registerBackgroundMessageHandler(MessageHandler handler) async {
    if (Platform.isIOS) {
      return;
    }

    if (!_bgHandlerInitialized) {
      _bgHandlerInitialized = true;
      HashMap<String, dynamic> list = HashMap();
      list[BACKGROUND_HANDLER] = PluginUtilities.getCallbackHandle(callbackDispatcher)!.toRawHandle();
      list[ON_PUSH_RECEIVE_BACKGROUND] = PluginUtilities.getCallbackHandle(handler)!.toRawHandle();
      await _channel.invokeMethod(START_BACKGROUND_ISOLATE, list);
    }
  }
}
